﻿using UnityEngine;
using System.Collections;

public enum GameState {InGame, GameOver, InMenu}

public class Game : MonoBehaviour {
	
	private const int 	houseDist 	= 8;
	private const int 	houseGen	= 8;
	private const float aspectRatio	= 16f/10f;
	
	public static Game handler;
	
	public int[] 	 screenResX;
	
	//percentage of buildings with enemies on the roof
	public int 		 weaponizeFactor = 30;
	//number of moved pixel before factor is incremented by 1
	public int       weaponizeAdd	 = 10; 
	//max. weaponize factor
	public int       weaponizeMax	 = 90;
	
	public Transform genFloor;
	public Transform genWindow;
	public Transform genRoof;
	public Transform genRoofFire;
	public Transform housePrefab;
	public Transform houseInitPrefab;
	
	public Transform playerPrefab;
	public Pixel 	 player;
	
	public Transform genRocketFire;
		
	public Mover 		mover;
	public Score 		goScore;
	public GameObject 	goTitle;
	
	public GameState gameState = GameState.InMenu;
	
	public  int enemyCount		 	= 0;
	public  int score				= 0;
	
	private int weaponizeFactorSave;
	private int generateHouseX 		= 0;
	private int currentResolution 	= 0;
	
	void Awake () {
		//assign global handler for easy access from other classes
		handler = this;
		
		//save weaponize factor as it will be overwritten later on
		weaponizeFactorSave = weaponizeFactor;
		
		NextScreenResolution();
			
	}
	
	void Start () {
		
		InitGame();
	}
	
	void Update () {
	
		bool keyEnterPressed  = Input.GetKeyDown(KeyCode.Return);
		bool keyEscapePressed = Input.GetKeyDown(KeyCode.Escape);
		
		//handle game state
		switch (gameState) {
			case GameState.GameOver:
				if (keyEnterPressed || keyEscapePressed) InitGame();
				break;
			case GameState.InGame:
				goTitle.SetActive(false);
				mover.dir = Vector2.right;
				if (keyEscapePressed) InitGame();
				break;
			case GameState.InMenu:
				goTitle.SetActive(true);			
				if (keyEnterPressed)  gameState = GameState.InGame;			
				if (keyEscapePressed) Application.Quit();
				break;		
		}
	
		//handle resolutions
		if (Input.GetKeyDown(KeyCode.F1)) NextScreenResolution();
		
		
		//set weaponize factor
		weaponizeFactor = weaponizeFactorSave + (int)mover.transform.position.x / weaponizeAdd;
		if (weaponizeFactor > weaponizeMax) weaponizeFactor = weaponizeMax;
		
		//increase factor if too few weaponized houses 
		if (weaponizeFactor/10 > enemyCount+1) weaponizeFactor += 30; 
		
		//decrease factor if too many weaponized houses
		if (weaponizeFactor/5 < enemyCount+1) weaponizeFactor -= 30;	
	
		if (weaponizeFactor > 100) weaponizeFactor = 100;
		
		//add additional houses as game scrolls on
		if (mover.transform.position.x > generateHouseX){
			
			generateHouseX += houseDist;
			Instantiate(housePrefab, new Vector3(generateHouseX+houseGen*houseDist,0,0), Quaternion.identity);	
		}	
	}
	
	public void InitGame() {
			
		gameState = GameState.InMenu;
		
		//reset globals
		score 		= 0;
		enemyCount 	= 0;
		
		//reset mover
		mover.pos   = Vector2.zero;
		mover.dir   = Vector2.zero;
		mover.transform.position = Vector3.zero;
		
		//clear pixel
		GameObject[] goAll = GameObject.FindGameObjectsWithTag("Respawn");
		foreach (GameObject goDelete in goAll) {
			Destroy(goDelete);
		}		
		
		//add player
		player = Instantiate(playerPrefab, transform.position, Quaternion.identity) as Pixel;
		
		generateHouseX = 0;
		
//		//generate houses (without weapons at the beginning)
//		weaponizeFactor = 0;
//		for (int i=0;i<=houseGen;i++) {
//			Instantiate(housePrefab, new Vector3(i*houseDist,0,0), Quaternion.identity);
//		}
		Instantiate(houseInitPrefab);
		
		//hide score
		goScore.gameObject.SetActive(false);
		
	}
	
	public void OnGameOver( int pixelPos ) {
		
		gameState = GameState.GameOver;
		
		//stop movement
		mover.dir = Vector2.zero;
		//show score
		goScore.gameObject.SetActive(true);
		goScore.SetScore( score * 20 + pixelPos * 10 ); 		
	}
	
	void NextScreenResolution( ) {	
		Screen.SetResolution(screenResX[currentResolution], (int)(screenResX[currentResolution]*10/16), false);	
		currentResolution++;
		if (currentResolution >= screenResX.Length) currentResolution = 0;
	}
}
