﻿using UnityEngine;
using System.Collections;

public enum SoundMode { DontPlay, OnStart, OnDeath }; 

public class Pixel : MonoBehaviour {
	
	public bool 	isPlayer;	
	
	public Vector2 	pos;
	public Vector2 	dir;
	
	public float   	speed 	 	 	= 10;
	public float    speedRandom  	= 0;
	public float	friction	 	= 1; //default no friction
	
	public float   	destroyTimer 	= 0;
	public float	destroyRandom	= 1;
	
	public int 		hitPoints	 	= 1;
	public bool     adjustCollider	= false;
	
	public Sound    defaultSound;
	public SoundMode playSound		= SoundMode.DontPlay;
	
	public float    zPosition		= 0;
	
	void Start() {
		
		pos = new Vector2(transform.localPosition.x, transform.localPosition.y);
		if (destroyTimer > 0) Destroy(gameObject, destroyTimer + Random.Range(-destroyRandom,destroyRandom));
	
		speed = Random.Range(speed-speedRandom, speed+speedRandom);
		
		//adjust collider
		if (adjustCollider && (dir != Vector2.zero)) {
		
			BoxCollider colli = GetComponent<BoxCollider>();
					
			//adjust collider size in one direction for better coll detection			
			if (Mathf.Abs(dir.x) >= Mathf.Abs(dir.y)) {
				colli.center = new Vector3(0.5f * -Mathf.Sign(dir.x),0,0);
				colli.size	 = new Vector3(1.5f * Mathf.Sign(dir.x),1,5);
			}
			else {
				colli.center = new Vector3(0,0.5f * -Mathf.Sign(dir.y),0);
				colli.size	 = new Vector3(1,1.5f * Mathf.Sign(dir.y),5);
			}
		}
		
		//play sound
		if (playSound == SoundMode.OnStart) PlaySound( defaultSound );
	}
	
	// Update is called once per frame
	void Update () {
	
		//movement
		pos = pos + dir * Time.deltaTime * speed;
		speed *= Mathf.Pow(friction, Time.deltaTime);
		
		transform.localPosition = new Vector3(Mathf.Round(pos.x), Mathf.Round(pos.y), zPosition);		
	}
	
	void OnTriggerEnter(Collider other) {
	
		hitPoints--;
		if (hitPoints <= 0) {
			if (!isPlayer) Game.handler.score++;
			Destroy(gameObject);
			if (playSound == SoundMode.OnDeath) PlaySound( defaultSound );
			SendMessage("OnDeath",null,SendMessageOptions.DontRequireReceiver);
		}
    }
	
	public void PlaySound( Sound sound = null ) {
		if (sound == null) sound = defaultSound;
		Instantiate(sound, transform.position, Quaternion.identity);
	}		
}
