﻿using UnityEngine;
using System.Collections;
using System;

public class Player : MonoBehaviour {
	
	private enum Axis {Zero, X, Y};
	
	public Pixel 	shotType;
	public Pixel    playerDeadPrefab;
	
	public Vector2	minPos;
	public Vector2	maxPos;
	
	private Vector2 oldDir = Vector2.zero;
	
	private Pixel	pixel;
	private Shooter shooter;
	
	private float axisXMoveOld = 0;
	private float axisYMoveOld = 0;
	private float axisXFireOld = 0;
	private float axisYFireOld = 0;	
	private Axis  lastMove = Axis.Zero;
	private Axis  lastFire = Axis.Zero;

	void Awake() { 
		pixel 	= GetComponent<Pixel>(); 
		shooter	= GetComponent<Shooter>();
		transform.parent = Game.handler.mover.transform;
	}
	
	// Update is called once per frame
	void Update () {
		//skip input if in menu state
		if (Game.handler.gameState != GameState.InGame) return;
		
		float axisXMove = Input.GetAxis("Horizontal");
		float axisYMove = Input.GetAxis("Vertical");
		float axisXFire = Input.GetAxis("HFire");
		float axisYFire = Input.GetAxis("VFire");	
		
		if (Math.Sign(axisXMove) != axisXMoveOld) lastMove = Axis.X;
		if (Math.Sign(axisYMove) != axisYMoveOld) lastMove = Axis.Y;		
		if (Math.Sign(axisXFire) != axisXFireOld) lastFire = Axis.X;
		if (Math.Sign(axisYFire) != axisYFireOld) lastFire = Axis.Y;
		
		axisXMoveOld = Math.Sign(axisXMove);
		axisYMoveOld = Math.Sign(axisYMove);		
		axisXFireOld = Math.Sign(axisXFire);		
		axisYFireOld = Math.Sign(axisYFire);		
		
		oldDir = pixel.dir;

		//movement
		pixel.dir = GetInputDir( axisXMove, axisYMove, lastMove );
		
		//boundary for movement 
		if (pixel.dir.x < 0 && Mathf.Round(pixel.pos.x) <= minPos.x) pixel.dir.x = 0;
		if (pixel.dir.x > 0 && Mathf.Round(pixel.pos.x) >= maxPos.x) pixel.dir.x = 0;
		if (pixel.dir.y < 0 && Mathf.Round(pixel.pos.y) <= minPos.y) pixel.dir.y = 0;
		if (pixel.dir.y > 0 && Mathf.Round(pixel.pos.y) >= maxPos.y) pixel.dir.y = 0;		
		
		//move instantly on change of direction (feels more responsive)
		if (oldDir != pixel.dir){
			
			pixel.pos.x = Mathf.Round(pixel.pos.x) + pixel.dir.x * 0.51f;
			pixel.pos.y = Mathf.Round(pixel.pos.y) + pixel.dir.y * 0.51f;	
		}
		
		//shoot
		Vector2 fireDir = GetInputDir( axisXFire, axisYFire, lastFire );
		if (fireDir != Vector2.zero) shooter.Shoot(fireDir,transform.parent);
	}
		
	Vector2 GetInputDir( float axisX, float axisY, Axis lastChange ) {
		
		Vector2 returnX = new Vector2(Mathf.Sign(axisX),0);
		Vector2 returnY = new Vector2(0,Mathf.Sign(axisY));
		
		//no axis pressed
		if (axisY == 0 && axisX == 0) return Vector2.zero;
		
		//only one axis is pressed
		if (axisY == 0) return returnX;
		if (axisX == 0) return returnY;
		
		//both axis are pressed
		//joystick/analogue input: return axis with greater amplitude
		if (Mathf.Abs(axisX) > Mathf.Abs(axisY)) return returnX;
		if (Mathf.Abs(axisY) > Mathf.Abs(axisX)) return returnY;
		
		//keyboard/digital input: return axis which changed previously
		if (lastChange == Axis.X) return returnX;
		return returnY;	
	}
	
	void OnDeath() {	
		Instantiate(playerDeadPrefab,transform.position,Quaternion.identity);
		Game.handler.OnGameOver( (int)transform.position.x );
	}	
}
